@@ -1,5 +1,15 @@
 package Data::Fetch;
 
+# Author Nigel Horne: njh@bandsman.co.uk
+# Copyright (C) 2016, Nigel Horne
+
+# Usage is subject to licence terms.
+# The licence terms of this software are as follows:
+# Personal single user, single computer use: GPL2
+# All other users (including Commercial, Charity, Educational, Government)
+#	must apply in writing for a licence for use from Nigel Horne at the
+#	above e-mail.
+
 # use 5.12.0;	# Threads before that are apparently not good
 use 5.10.0;	# Earliest version that Coro works with
 use strict;
@@ -13,11 +23,11 @@ Data::Fetch - give advance warning that you'll be needing a value
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 SYNOPSIS
 
@@ -57,7 +67,8 @@ Takes two mandatory parameters:
     object - the object you'll be sending the message to
     message - the message you'll be sending
 
-Takes one mandatory parameter:
+Takes one optional parameter:
+
     arg - passes this argument to the message
 
 =cut
@@ -69,8 +80,11 @@ sub prime {
 	return unless($args{'object'} && $args{'message'});
 
 	my $object = $args{'object'} . '->' . $args{'message'};
+	if(my $a = $args{arg}) {
+		$object .= "($a)"
+	}
 
-	if($self->{values}->{$object}) {
+	if($self->{values} && $self->{values}->{$object} && $self->{values}->{$object}->{status}) {
 		return $self;
 	}
 	# $self->{values}->{$object}->{thread} = threads->create(sub {
@@ -82,6 +96,7 @@ sub prime {
 		# return eval '$o->$m()';
 	# }, $args{'object'}, $args{'message'}, $args{'arg'});
 
+	$self->{values}->{$object}->{status} = 'running';
 	$self->{values}->{$object}->{thread} = async {
 		my $o = $args{object};
 		my $m = $args{message};
@@ -101,6 +116,10 @@ Retrieve get a value you've primed.  Takes two mandatory parameters:
     object - the object you'll be sending the message to
     message - the message you'll be sending
 
+Takes one optional parameter:
+
+    arg - passes this argument to the message
+
 =cut
 
 sub get {
@@ -110,17 +129,25 @@ sub get {
 	return unless($args{'object'} && $args{'message'});
 
 	my $object = $args{'object'} . '->' . $args{'message'};
+	if(my $a = $args{arg}) {
+		$object .= "($a)"
+	}
 
-	if($self->{values}->{$object}->{value}) {
+	if(!defined($self->{values}->{$object}->{status})) {
+		my @call_details = caller(0);
+		die "Need to prime before getting at line ", $call_details[2], ' of ', $call_details[1];
+	}
+	if($self->{values}->{$object}->{status} eq 'complete') {
 		return $self->{values}->{$object}->{value};
 	}
-	if($self->{values}->{$object}->{thread}) {
+	if($self->{values}->{$object}->{status} eq 'running') {
 		my $rc = $self->{values}->{$object}->{thread}->join();
+		$self->{values}->{$object}->{status} = 'complete';
 		delete $self->{values}->{$object}->{thread};
 		# $self->{values}->{$object}->{thread} = undef;	# ????
 		return $self->{values}->{$object}->{value} = $rc;
 	}
-	die "Need to prime before getting";
+	die 'Unknown status: ', $self->{values}->{$object}->{status};
 }
 
 sub DESTROY {
@@ -146,7 +173,9 @@ Nigel Horne, C<< <njh at bandsman.co.uk> >>
 
 =head1 BUGS
 
-Can't pass more than one argument to the message
+Can't pass more than one argument to the message.
+
+I would not advise using this to call messages that change values in the object.
 
 Changing a value between prime and get will not necessarily get you the data you want. That's the way it works
 and isn't going to change.
